@@ -1,12 +1,13 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
 
 #include "arp.h"
 #include "config.h"
+#include "debug.h"
 #include "ethernet.h"
 #include "ip.h"
 #include "macros.h"
@@ -39,11 +40,14 @@ void ArpInit( void ) //(extern)
 }
 
 //send an ARP request
-//must be called with interrupts disabled
 static void ArpSendRequest( unsigned char * pIp )
 {
   struct ArpPacket ArpRequest;
 
+  debug_arp_printf( "send req ip=%u.%u.%u.%u",
+                    ArpRequest.ArpHdr.DestIp[0], ArpRequest.ArpHdr.DestIp[1], 
+                    ArpRequest.ArpHdr.DestIp[2], ArpRequest.ArpHdr.DestIp[3] );
+
   //build ARP request
   ArpRequest.ArpHdr.HwType = htons( 0x0001 ); //ethernet
   ArpRequest.ArpHdr.ProtoType = htons( 0x0800 ); //IP
@@ -61,8 +65,7 @@ static void ArpSendRequest( unsigned char * pIp )
   EthernetSend( (unsigned char *)&ArpRequest, sizeof( ArpRequest ) );
 }
 
-//tick procedure - call every 200ms from timer interrupt
-//must be called with interrupts disabled
+//tick procedure - call every 200ms
 void ArpTick200( void ) //(extern)
 {
   unsigned char i;
@@ -90,7 +93,6 @@ void ArpTick200( void ) //(extern)
 }
 
 //process a received ARP packet
-//must be called with interrupts disabled
 void ArpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct ArpPacket * pArpPack;
@@ -131,6 +133,13 @@ void ArpRecv( unsigned char * pData, unsigned short Length ) //(extern)
     ip_cpy( ArpReply.ArpHdr.SrcIp, ConfigIp ); //own IP
     mac_cpy( ArpReply.ArpHdr.DestMac, pArpPack->ArpHdr.SrcMac ); //requestor's MAC
     ip_cpy( ArpReply.ArpHdr.DestIp, pArpPack->ArpHdr.SrcIp ); //requestor's IP
+
+    debug_arp_printf( "recv req src=%02X:%02X:%02X:%02X:%02X:%02X ip=%u.%u.%u.%u",
+                      ArpReply.ArpHdr.SrcMac[0], ArpReply.ArpHdr.SrcMac[1], ArpReply.ArpHdr.SrcMac[2],
+                      ArpReply.ArpHdr.SrcMac[3], ArpReply.ArpHdr.SrcMac[4], ArpReply.ArpHdr.SrcMac[5],
+                      ArpReply.ArpHdr.SrcIp[0], ArpReply.ArpHdr.SrcIp[1], 
+                      ArpReply.ArpHdr.SrcIp[2], ArpReply.ArpHdr.SrcIp[3] );
+
     //sent ARP reply
     mac_cpy( ArpReply.EthHdr.Dest, ArpReply.ArpHdr.DestMac ); //ethernet destination address
     ArpReply.EthHdr.Type = htons( 0x0806 ); //ethernet packet type: ARP
@@ -144,6 +153,13 @@ void ArpRecv( unsigned char * pData, unsigned short Length ) //(extern)
       ip_eq( pArpPack->ArpHdr.DestIp, ConfigIp ) ) //own IP address
   {
     unsigned char i;
+
+    debug_arp_printf( "recv reply src=%02X:%02X:%02X:%02X:%02X:%02X ip=%u.%u.%u.%u",
+                      pArpPack->ArpHdr.SrcMac[0], pArpPack->ArpHdr.SrcMac[1], pArpPack->ArpHdr.SrcMac[2],
+                      pArpPack->ArpHdr.SrcMac[3], pArpPack->ArpHdr.SrcMac[4], pArpPack->ArpHdr.SrcMac[5],
+                      pArpPack->ArpHdr.SrcIp[0], pArpPack->ArpHdr.SrcIp[1], 
+                      pArpPack->ArpHdr.SrcIp[2], pArpPack->ArpHdr.SrcIp[3] );
+
     //search IP in ARP tabale
     for( i = 0; i < count( ArpTab ); i++ )
       if( (ArpTab[i].Flags & ArpTabFlagInUse) &&
@@ -169,7 +185,6 @@ void ArpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 
 //lookup the MAC for an IP address
 //returns 0x00 in case of success, 0x01 if the MAC address is unknown
-//must be called with interrupts disabled
 unsigned char ArpLookup( unsigned char * pIp, unsigned char * pMac ) //(extern)
 {
   unsigned char i, j;
