@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -22,7 +22,7 @@ unsigned short ConfigS8pPort = 8; //disabled if 0 (extern)
 unsigned char ConfigS8pKey[16] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }; //(extern)
 
 //port for webserver
-unsigned short ConfigWebPort = 80; //disabled if 0 (extern)
+unsigned short ConfigHttpPort = 80; //disabled if 0 (extern)
 
 //default output state
 unsigned char ConfigDefOut = 0x00; //(extern)
