@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -9,6 +9,7 @@
 #include <avr/eeprom.h>
 
 #include "config.h"
+#include "debug.h"
 #include "eeprom.h"
 #include "macros.h"
 #include "output.h"
@@ -34,8 +35,8 @@ unsigned char EepromWriteCnt; //number of bytes still to write
 #define EEPROM_ADDR_GW (EEPROM_ADDR_MASK + 2 * sizeof( ConfigMask ))
 #define EEPROM_ADDR_S8P_PORT (EEPROM_ADDR_GW + 2 * sizeof( ConfigGw ))
 #define EEPROM_ADDR_S8P_KEY (EEPROM_ADDR_S8P_PORT + 2 * sizeof( ConfigS8pPort ))
-#define EEPROM_ADDR_WEB_PORT (EEPROM_ADDR_S8P_KEY + 2 * sizeof( ConfigS8pKey ))
-#define EEPROM_ADDR_DEF_OUT (EEPROM_ADDR_WEB_PORT + 2 * sizeof( ConfigWebPort ))
+#define EEPROM_ADDR_HTTP_PORT (EEPROM_ADDR_S8P_KEY + 2 * sizeof( ConfigS8pKey ))
+#define EEPROM_ADDR_DEF_OUT (EEPROM_ADDR_HTTP_PORT + 2 * sizeof( ConfigHttpPort ))
 #define EEPROM_ADDR_FreeFromHere (EEPROM_ADDR_DEF_OUT + 2 * sizeof( ConfigDefOut ))
 
 //get configuration from EEPROM - helper function
@@ -76,7 +77,6 @@ static inline unsigned char EepromGetConfigHelp( unsigned short SrcAddr, unsigne
 }
 
 //get configuration from EEPROM
-//must be called with interrupts disabled
 void EepromGetConfig( void ) //(extern)
 {
   //get MAC
@@ -96,7 +96,7 @@ void EepromGetConfig( void ) //(extern)
   EepromGetConfigHelp( EEPROM_ADDR_S8P_KEY, ConfigS8pKey, sizeof( ConfigS8pKey ) );
 
   //get port for webserver
-  EepromGetConfigHelp( EEPROM_ADDR_WEB_PORT, (unsigned char *)&ConfigWebPort, sizeof( ConfigWebPort ) );
+  EepromGetConfigHelp( EEPROM_ADDR_HTTP_PORT, (unsigned char *)&ConfigHttpPort, sizeof( ConfigHttpPort ) );
 
   //get default output state
   if( EepromGetConfigHelp( EEPROM_ADDR_DEF_OUT, &ConfigDefOut, sizeof( ConfigDefOut ) ) == 0x00 )
@@ -165,12 +165,12 @@ void EepromTask( void ) //(extern)
       EepromWriteFlags = EepromWriteFlagActive;
     }
     //port for webserver needs to be written
-    else if( EepromNeedWrite & EepromNeedWriteWebPort )
+    else if( EepromNeedWrite & EepromNeedWriteHttpPort )
     {
-      EepromNeedWrite &= ~EepromNeedWriteWebPort;
-      pEepromWriteSrc = (unsigned char *)&ConfigWebPort;
-      EepromWriteDestAddr = EEPROM_ADDR_WEB_PORT;
-      EepromWriteCnt = sizeof( ConfigWebPort );
+      EepromNeedWrite &= ~EepromNeedWriteHttpPort;
+      pEepromWriteSrc = (unsigned char *)&ConfigHttpPort;
+      EepromWriteDestAddr = EEPROM_ADDR_HTTP_PORT;
+      EepromWriteCnt = sizeof( ConfigHttpPort );
       EepromWriteFlags = EepromWriteFlagActive;
     }
     //if default output state needs to be written
@@ -185,6 +185,8 @@ void EepromTask( void ) //(extern)
     //nothing needs to be written
     else
       return;
+
+    debug_eeprom_printf( "write adr=0x%04X len=%u", EepromWriteDestAddr, EepromWriteCnt );
   }
 
   //EEPROM still busy
@@ -198,7 +200,12 @@ void EepromTask( void ) //(extern)
   //get address in EEPROM and data to write
   EEAR = EepromWriteDestAddr++;
   if( EepromWriteFlags & EepromWriteFlagCheck ) //check byte
+  {
     EEDR = ~*(pEepromWriteSrc++); //invert and advance source pointer
+    EepromWriteCnt--; //count down number of bytes to write
+    if( EepromWriteCnt <= 0 ) //finished
+      EepromWriteFlags &= ~EepromWriteFlagActive;
+  }
   else //not check byte
     EEDR = *pEepromWriteSrc; //use data byte as it is
   EepromWriteFlags ^= EepromWriteFlagCheck; //toggle check byte bit
