@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -15,12 +15,11 @@
 #define EepromNeedWriteGw 0x08
 #define EepromNeedWriteS8pPort 0x10
 #define EepromNeedWriteS8pKey 0x20
-#define EepromNeedWriteWebPort 0x40
+#define EepromNeedWriteHttpPort 0x40
 #define EepromNeedWriteDefOut 0x80
 extern unsigned char EepromNeedWrite;
 
 //get configuration from EEPROM
-//must be called with interrupts disabled
 extern void EepromGetConfig( void );
 
 //task function to do the work - call from main loop
