@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -8,6 +8,7 @@
 #include <string.h>
 #include <avr/pgmspace.h>
 
+#include "debug.h"
 #include "http.h"
 #include "macros.h"
 #include "output.h"
@@ -26,10 +27,12 @@
 //files available using HTTP
 #define HTTP_FILE_NOT_FOUND 0 //file not found file
 #define HTTP_FILE_INDEX 1 //index file
-#define HTTP_FILE_ON 2 //turn on links (+0..+7)
-#define HTTP_FILE_OFF 10 //turn off links (+0..+7)
-#define HTTP_FILE_IMG1 18 //image: on
-#define HTTP_FILE_IMG0 19 //image: off
+#define HTTP_FILE_ON_ALL 2 //turn on all link
+#define HTTP_FILE_ON_SINGLE 3 //turn on single links (+0..+7)
+#define HTTP_FILE_OFF_ALL 11 //turn off all link
+#define HTTP_FILE_OFF_SINGLE 12 //turn off single links (+0..+7)
+#define HTTP_FILE_IMG1 20 //image: on
+#define HTTP_FILE_IMG0 21 //image: off
 
 //table with HTTP connections
 struct HttpConnection
@@ -139,6 +142,11 @@ const char PROGMEM HttpIndex[] =
 "          <td align=\"center\"><img src=\"/img\x98\" alt=\"\x88\x88\x88\">&nbsp;\x88\x88\x88</td>\r\n"
 "          <td align=\"center\"><a href=\"/on8\">on</a>&nbsp;&nbsp;<a href=\"/off8\">&nbsp;off</a></td>\r\n"
 "        </tr>\r\n"
+"        <tr>\r\n"
+"          <td align=\"center\">all</td>\r\n"
+"          <td align=\"center\">&nbsp;</td>\r\n"
+"          <td align=\"center\"><a href=\"/on\">on</a>&nbsp;&nbsp;<a href=\"/off\">&nbsp;off</a></td>\r\n"
+"        </tr>\r\n"
 "      </table>\r\n"
 "      <br><br>\r\n"
 "      <a href=\"/\">refresh</a>\r\n"
@@ -248,14 +256,20 @@ static void HttpParseFile( struct HttpConnection * pConn )
   //index file
   if( pConn->RcvBufLen == 1 && strncasecmp( pConn->RcvBuf, "/", 1 ) == 0 )
     pConn->File = HTTP_FILE_INDEX;
-  //turn on link
+  //turn on all link
+  else if( pConn->RcvBufLen == 3 && strncasecmp( pConn->RcvBuf, "/on", 3 ) == 0 )
+    pConn->File = HTTP_FILE_ON_ALL;
+  //turn on single link
   else if( pConn->RcvBufLen == 4 && strncasecmp( pConn->RcvBuf, "/on", 3 ) == 0 &&
            pConn->RcvBuf[3] >= '1' && pConn->RcvBuf[3] <= '8' )
-    pConn->File = HTTP_FILE_ON + pConn->RcvBuf[3] - '1';
-  //turn off link
+    pConn->File = HTTP_FILE_ON_SINGLE + pConn->RcvBuf[3] - '1';
+  //turn off all link
+  else if( pConn->RcvBufLen == 4 && strncasecmp( pConn->RcvBuf, "/off", 4 ) == 0 )
+    pConn->File = HTTP_FILE_OFF_ALL;
+  //turn off single link
   else if( pConn->RcvBufLen == 5 && strncasecmp( pConn->RcvBuf, "/off", 4 ) == 0 &&
            pConn->RcvBuf[4] >= '1' && pConn->RcvBuf[4] <= '8' )
-    pConn->File = HTTP_FILE_OFF + pConn->RcvBuf[4] - '1';
+    pConn->File = HTTP_FILE_OFF_SINGLE + pConn->RcvBuf[4] - '1';
   //image: on
   else if( pConn->RcvBufLen == 5 && strncasecmp( pConn->RcvBuf, "/img1", 5 ) == 0 )
     pConn->File = HTTP_FILE_IMG1;
@@ -279,27 +293,37 @@ static void HttpProcessRequest( struct HttpConnection * pConn )
       switch( pConn->File )
       {
 
-        case HTTP_FILE_ON+0:
-        case HTTP_FILE_ON+1:
-        case HTTP_FILE_ON+2:
-        case HTTP_FILE_ON+3:
-        case HTTP_FILE_ON+4:
-        case HTTP_FILE_ON+5:
-        case HTTP_FILE_ON+6:
-        case HTTP_FILE_ON+7:
-          OutputChange( 1, pConn->File - HTTP_FILE_ON + 1 ); //turn on selected output
+        case HTTP_FILE_ON_ALL:
+          OutputChangeAll( 1 ); //turn on all outputs
           pConn->File = HTTP_FILE_INDEX; //"symlink" to index file
           break;
 
-        case HTTP_FILE_OFF+0:
-        case HTTP_FILE_OFF+1:
-        case HTTP_FILE_OFF+2:
-        case HTTP_FILE_OFF+3:
-        case HTTP_FILE_OFF+4:
-        case HTTP_FILE_OFF+5:
-        case HTTP_FILE_OFF+6:
-        case HTTP_FILE_OFF+7:
-          OutputChange( 0, pConn->File - HTTP_FILE_OFF + 1 ); //turn off selected output
+        case HTTP_FILE_ON_SINGLE+0:
+        case HTTP_FILE_ON_SINGLE+1:
+        case HTTP_FILE_ON_SINGLE+2:
+        case HTTP_FILE_ON_SINGLE+3:
+        case HTTP_FILE_ON_SINGLE+4:
+        case HTTP_FILE_ON_SINGLE+5:
+        case HTTP_FILE_ON_SINGLE+6:
+        case HTTP_FILE_ON_SINGLE+7:
+          OutputChange( 1, pConn->File - HTTP_FILE_ON_SINGLE + 1 ); //turn on selected output
+          pConn->File = HTTP_FILE_INDEX; //"symlink" to index file
+          break;
+
+        case HTTP_FILE_OFF_ALL:
+          OutputChangeAll( 0 ); //turn off all outputs
+          pConn->File = HTTP_FILE_INDEX; //"symlink" to index file
+          break;
+
+        case HTTP_FILE_OFF_SINGLE+0:
+        case HTTP_FILE_OFF_SINGLE+1:
+        case HTTP_FILE_OFF_SINGLE+2:
+        case HTTP_FILE_OFF_SINGLE+3:
+        case HTTP_FILE_OFF_SINGLE+4:
+        case HTTP_FILE_OFF_SINGLE+5:
+        case HTTP_FILE_OFF_SINGLE+6:
+        case HTTP_FILE_OFF_SINGLE+7:
+          OutputChange( 0, pConn->File - HTTP_FILE_OFF_SINGLE + 1 ); //turn off selected output
           pConn->File = HTTP_FILE_INDEX; //"symlink" to index file
           break;
 
@@ -388,7 +412,7 @@ static void HttpProcessRequest( struct HttpConnection * pConn )
 //get a variable
 //pBuffer must provide space for at least 16 characters
 //returns the length of the variable
-static unsigned char HttpGetVariable( unsigned char VarNo, char * pBuffer )
+static unsigned char HttpGetVariable( unsigned char VarNo, char * pBuffer, unsigned char MaxLen )
 {
   switch( VarNo )
   {
@@ -402,6 +426,8 @@ static unsigned char HttpGetVariable( unsigned char VarNo, char * pBuffer )
     case 0x86:
     case 0x87:
     case 0x88:
+      if( MaxLen < 3 )
+        return 0;
       if( OutputGet( VarNo - 0x80 ) )
       {
         memcpy( pBuffer, "ON", 2 );
@@ -422,6 +448,8 @@ static unsigned char HttpGetVariable( unsigned char VarNo, char * pBuffer )
     case 0x96:
     case 0x97:
     case 0x98:
+      if( MaxLen < 1 )
+        return 0;
       if( OutputGet( VarNo - 0x90 ) )
         pBuffer[0] = '1';
       else
@@ -458,6 +486,8 @@ void HttpConnect( unsigned char ConnNo )
   //get pointer to connection
   pConn = &HttpConns[i];
 
+  debug_http_printf( "accept no=%u", ConnNo );
+
   //put new connection into table
   pConn->ConnNo = ConnNo;
   pConn->RcvCnt = 0;
@@ -481,6 +511,8 @@ void HttpClose( unsigned char ConnNo )
   //get pointer to connection
   pConn = &HttpConns[i];
 
+  debug_http_printf( "close no=%u", ConnNo );
+
   //drop connection from table
   pConn->ConnNo = 0xFF;
 }
@@ -492,7 +524,7 @@ unsigned short HttpSend( unsigned char ConnNo, unsigned long Pos, unsigned char
   unsigned char i, var, VarLen, VarPos;
   struct HttpConnection * pConn;
   unsigned short len, j;
-  uint16_t src; //progmem pointer
+  uint16_t src, srcTmp; //progmem pointer
   char chr, * dest, VarBuffer[16];
 
   //find connection in table
@@ -528,11 +560,11 @@ unsigned short HttpSend( unsigned char ConnNo, unsigned long Pos, unsigned char
   {
     //get variable
     var = chr;
-    VarLen = HttpGetVariable( var, VarBuffer );
+    VarLen = HttpGetVariable( var, VarBuffer, sizeof( VarBuffer ) );
     //get position in variable
-    for( VarPos = 0; src > pConn->pSndData; VarPos++, src-- )
-      if( (char)pgm_read_byte_near( src ) != var ) //if normal character of other variable, we found the begin of the variable
-        { VarPos++; break; }
+    for( VarPos = 0, srcTmp = src - 1; srcTmp > pConn->pSndData; VarPos++, srcTmp-- )
+      if( (char)pgm_read_byte_near( srcTmp ) != var ) //if normal character or other variable, we found the begin of the variable
+        break;
   }
   //copy data to buffer
   dest = pBuffer;
@@ -548,7 +580,7 @@ unsigned short HttpSend( unsigned char ConnNo, unsigned long Pos, unsigned char
       {
         //get variable
         var = chr;
-        VarLen = HttpGetVariable( var, VarBuffer );
+        VarLen = HttpGetVariable( var, VarBuffer, sizeof( VarBuffer ) );
         VarPos = 0;
       }
       //copy next character of variable
@@ -560,8 +592,8 @@ unsigned short HttpSend( unsigned char ConnNo, unsigned long Pos, unsigned char
     //normal character
     else
     {
-      *dest = chr;
-      VarPos = 0; //reset variable position (needed if same variable appears two times in a row)
+      var = 0; //not a variable
+      *dest = chr; //copy character
     }
     //next character
     src++;
@@ -569,6 +601,7 @@ unsigned short HttpSend( unsigned char ConnNo, unsigned long Pos, unsigned char
   } //for( j ...
 
   //return length of data in buffer
+  debug_http_printf( "send no=%u pos=%lu len=%u", ConnNo, Pos, len );
   return len;
 }
 
@@ -627,6 +660,7 @@ unsigned short HttpReceived( unsigned char ConnNo, unsigned long Pos, unsigned c
           pConn->PreHttp1 = 1; //older than HTTP 1.0
           pConn->File = HTTP_FILE_INDEX; //send index file
           pConn->RcvState = HTTP_RCV_DONE; //receiving completed
+          debug_http_printf( "get+end no=%u", pConn->ConnNo );
           pConn->RcvBufLen = 0; //empty receive buffer
           HttpProcessRequest( pConn ); //now process request
         }
@@ -635,6 +669,7 @@ unsigned short HttpReceived( unsigned char ConnNo, unsigned long Pos, unsigned c
           pConn->RcvBufLen--; //remove whitespace from buffer
           HttpParseCommand( pConn ); //parse command
           pConn->RcvState = HTTP_RCV_FILE; //now receive filename
+          debug_http_printf( "get no=%u", pConn->ConnNo );
           pConn->RcvBufLen = 0; //empty receive buffer
         }
         break;
@@ -647,6 +682,7 @@ unsigned short HttpReceived( unsigned char ConnNo, unsigned long Pos, unsigned c
           HttpParseFile( pConn ); //parse file
           pConn->PreHttp1 = 1; //older than HTTP 1.0
           pConn->RcvState = HTTP_RCV_DONE; //receiving completed
+          debug_http_printf( "file+end no=%u file=%u", pConn->ConnNo, pConn->File );
           pConn->RcvBufLen = 0; //empty receive buffer
           HttpProcessRequest( pConn ); //now process request
         }
@@ -656,6 +692,7 @@ unsigned short HttpReceived( unsigned char ConnNo, unsigned long Pos, unsigned c
           HttpParseFile( pConn ); //parse file
           pConn->PreHttp1 = 0; //HTTP 1.0 or newer
           pConn->RcvState = HTTP_RCV_HDR_END; //now receive header until end
+          debug_http_printf( "file no=%u file=%u", pConn->ConnNo, pConn->File );
           pConn->RcvBufLen = 0; //empty receive buffer
         }
         break;
@@ -674,6 +711,7 @@ unsigned short HttpReceived( unsigned char ConnNo, unsigned long Pos, unsigned c
             pConn->RcvBufLen == 4 ) //CR LF CR LF (or some other combination of 4 times CR or LF)
         {
           pConn->RcvState = HTTP_RCV_DONE; //receiving completed
+          debug_http_printf( "end no=%u", pConn->ConnNo );
           pConn->RcvBufLen = 0; //empty receive buffer
           HttpProcessRequest( pConn ); //now process request
         }
