@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -10,6 +10,7 @@
 #include "arp.h"
 #include "checksum.h"
 #include "config.h"
+#include "debug.h"
 #include "ethernet.h"
 #include "icmp.h"
 #include "ip.h"
@@ -19,7 +20,7 @@
 #include "udp.h"
 
 //timing parameters
-#define IpBufferTicksMax 50 //maximum age of buffered IP pakcet (in 200ms steps)
+#define IpBufferTicksMax 50 //maximum age of buffered IP packet (in 200ms steps)
 
 //buffers for IP packets to transmit
 // - used if MAC is unknown when packet shall be transmitted
@@ -35,7 +36,7 @@ struct IpBufferTable //table with buffers
   unsigned short PacketLength; //length of packet in buffer, 0 if no packet in this buffer
   unsigned char Ticks; //age of entry in 200ms steps
 } IpBufferTab[] =
-{ //put smaller buffers infont of larger buffers
+{ //put smaller buffers infront of larger buffers
   // - then short packets will use smaller buffers more often
   { IpBuffer0, sizeof( IpBuffer0 ), 0, 0 },
   { IpBuffer1, sizeof( IpBuffer1 ), 0, 0 },
@@ -43,8 +44,7 @@ struct IpBufferTable //table with buffers
   { IpBuffer3, sizeof( IpBuffer3 ), 0, 0 },
 };
 
-//tick procedure - call every 200ms from timer interrupt
-//must be called with interrupts disabled
+//tick procedure - call every 200ms
 void IpTick200( void ) //(extern)
 {
   unsigned char i;
@@ -62,7 +62,6 @@ void IpTick200( void ) //(extern)
 }
 
 //process a received IP packet
-//must be called with interrupts disabled
 void IpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct IpPacket * pIpPack;
@@ -124,6 +123,11 @@ void IpRecv( unsigned char * pData, unsigned short Length ) //(extern)
   if( Checksum( (unsigned char*)&pIpPack->IpHdr, sizeof( struct IpHeader ), 0x0000, 0x0000 ) != 0 )
     return;
 
+  debug_ip_printf( "recv src=%u.%u.%u.%u protocol=%u len=%u",
+                   pIpPack->IpHdr.Src[0], pIpPack->IpHdr.Src[1],
+                   pIpPack->IpHdr.Src[2], pIpPack->IpHdr.Src[3],
+                   pIpPack->IpHdr.Proto, Length );
+
   //branch according to protocol
   switch( pIpPack->IpHdr.Proto )
   {
@@ -144,7 +148,6 @@ void IpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 
 //send an IP packet
 //pData must point to a struct IpPacket with IpHdr.Proto and IpHdr.Dest already initialized
-//must be called with interrupts disabled
 void IpSend( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct IpPacket * pIpPack;
@@ -159,6 +162,11 @@ void IpSend( unsigned char * pData, unsigned short Length ) //(extern)
   //(this saves us from always casting pData)
   pIpPack = (struct IpPacket *)pData;
 
+  debug_ip_printf( "send dest=%u.%u.%u.%u protocol=%u len=%u",
+                   pIpPack->IpHdr.Dest[0], pIpPack->IpHdr.Dest[1],
+                   pIpPack->IpHdr.Dest[2], pIpPack->IpHdr.Dest[3],
+                   pIpPack->IpHdr.Proto, Length );
+
   //fill in header values
   pIpPack->IpHdr.Ver_HdrLen = 0x45;
   pIpPack->IpHdr.Tos = 0x00;
@@ -204,6 +212,7 @@ void IpSend( unsigned char * pData, unsigned short Length ) //(extern)
       memcpy( IpBufferTab[i].pBuffer, pData, Length );
       IpBufferTab[i].PacketLength = Length;
       IpBufferTab[i].Ticks = 0;
+      debug_ip_printf( "queued" );
       break;
     }
   }
@@ -212,7 +221,6 @@ void IpSend( unsigned char * pData, unsigned short Length ) //(extern)
 
 //a MAC address was discovered
 //called by ARP to notify IP
-//must be called with interrupts disabled
 void IpGotMac( unsigned char Ip[4], unsigned char Mac[6] ) //(extern)
 {
   unsigned char i;
@@ -226,6 +234,11 @@ void IpGotMac( unsigned char Ip[4], unsigned char Mac[6] ) //(extern)
       //convert pointer to IP packet
       pIpPack = (struct IpPacket *)IpBufferTab[i].pBuffer;
 
+      debug_ip_printf( "send queued dest=%u.%u.%u.%u protocol=%u len=%u",
+                       pIpPack->IpHdr.Dest[0], pIpPack->IpHdr.Dest[1],
+                       pIpPack->IpHdr.Dest[2], pIpPack->IpHdr.Dest[3],
+                       pIpPack->IpHdr.Proto, IpBufferTab[i].PacketLength );
+
       //destination is in own subnet
       if( (pIpPack->IpHdr.Dest[0] & ConfigMask[0]) == (ConfigIp[0] & ConfigMask[0]) &&
           (pIpPack->IpHdr.Dest[1] & ConfigMask[1]) == (ConfigIp[1] & ConfigMask[1]) &&
