@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -9,10 +9,12 @@
 #include <avr/wdt.h>
 
 #include "arp.h"
+#include "debug.h"
 #include "eeprom.h"
 #include "http.h"
 #include "keypad.h"
 #include "output.h"
+#include "random.h"
 #include "rtl8019.h"
 #include "s8p.h"
 #include "tcp.h"
@@ -23,12 +25,21 @@
 int main( void )
 {
   wdt_reset( );
+#ifdef DEBUG
+  wdt_disable( );
+#else
   wdt_enable( WDTO_60MS );
+#endif
   wdt_reset( );
 
   //initialize uart to be able to use stdio
   UartInit( );
 
+  debug_printf( "" );
+  debug_printf( "8BitAmEthernet" );
+
+  debug_init_printf( "init" );
+
   //initialize modules
   ArpInit( );
   HttpInit( );
@@ -39,19 +50,26 @@ int main( void )
   TcpInit( );
   TimingInit( );
 
+  debug_init_printf( "config" );
+
   //get configuration from EEPROM
   EepromGetConfig( );
 
   //enable interrupts
   sei( );
-  
+
+  debug_init_printf( "start" );
+
   //main loop
   while( 1 )
   {
     wdt_reset( );
     EepromTask( );
     KeypadTask( );
+    RandomTask( );
+    RtlTask( );
     S8pTask( );
+    TimingTask( );
   }
 
   return 0;
