@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -12,20 +12,20 @@
 extern void OutputInit( void );
 
 //get state of an output
-//must be called with interrupts turned off
 extern char OutputGet( unsigned char OutputNo );
 
 //change output
 //Action: 0=turn off, 1=turn on, -1=toggle
-//must be called with interrupts turned off
 extern void OutputChange( signed char Action, unsigned char OutputNo );
 
+//change all outputs
+//Action: 0=turn off, 1=turn on, -1=toggle
+extern void OutputChangeAll( signed char Action );
+
 //get output state
-//must be called with interrupts turned off
 extern unsigned char OutputGetState( void );
 
 //set output state
-//must be called with interrupts turned off
 extern void OutputSetState( unsigned char NewState );
 
 #endif //#ifdef INC_output
