@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -23,18 +23,21 @@
 #endif
 
 //entropy collected so far
-volatile unsigned long RandomEntropy = 0; //the entropy itself
-volatile unsigned char RandomEntropyCnt //number of times entropy was collected so far
-                       = sizeof( unsigned long ) * 8 - 1; //use first entropy collected as initial random seed (better than no seed at all)
+unsigned long RandomEntropy = 0; //the entropy itself
+unsigned char RandomEntropyCnt //number of times entropy was collected so far
+              = sizeof( unsigned long ) * 8 - 1; //use first entropy collected as initial random seed (better than no seed at all)
 
 //provide some entropy
-//must be called with interrupts disabled
 void RandomProvideEntropy( unsigned char Entropy ) //(extern)
 {
   //collect entropy
   RandomEntropy = RandomEntropy << 1 ^ (unsigned int)Entropy;
   RandomEntropyCnt++;
+}
 
+//task function to do the work - call from main loop
+void RandomTask( void ) //(extern)
+{
   //enough entropy was collected
   if( RandomEntropyCnt >= sizeof( unsigned long ) * 8 ) //one time entropy for every bit of RandomEntropy
   {
@@ -45,7 +48,6 @@ void RandomProvideEntropy( unsigned char Entropy ) //(extern)
 }
 
 //get random data
-//must be called with interrupts disabled
 void RandomGetData( unsigned char * pData, unsigned char Length ) //(extern)
 {
   unsigned long r;
