@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -9,11 +9,12 @@
 #define INC_random
 
 //provide some entropy
-//must be called with interrupts disabled
 extern void RandomProvideEntropy( unsigned char Entropy );
 
+//task function to do the work - call from main loop
+extern void RandomTask( void );
+
 //get random data
-//must be called with interrupts disabled
 extern void RandomGetData( unsigned char * pData, unsigned char Length );
 
 #endif //#ifndef INC_random
