@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -38,7 +38,7 @@
  *   "." is the appending operator
  *
  *   commands are identified by their first byte
- *   the status has alwys got the same first byte as the command
+ *   the status has always got the same first byte as the command
  *
  *   get output
  *     command: 0x00 7x<unused>
@@ -59,9 +59,9 @@
 
 #include <string.h>
 #include <avr/io.h>
-#include <avr/interrupt.h>
 
 #include "config.h"
+#include "debug.h"
 #include "macros.h"
 #include "output.h"
 #include "random.h"
@@ -74,14 +74,12 @@
 
 //states of S8P connections
 #define S8P_FREE 0 //free entry
-#define S8P_LOCKED 1 //locked entry, in use by non-interrupt-code
-#define S8P_LOCKED_FREE 2 //locked entry, please free when unlocking
-#define S8P_NEW 3 //new connection, waiting for challenge to be generated
-#define S8P_CHALLENGE 4 //sending challenge
-#define S8P_RESPONSE 5 //receiving response
-#define S8P_COMMAND 6 //waiting for execution of command
-#define S8P_ACKNOWLEDGE 7 //sending acknowledge
-#define S8P_CLOSE 8 //close connection
+#define S8P_NEW 1 //new connection, waiting for challenge to be generated
+#define S8P_CHALLENGE 2 //sending challenge
+#define S8P_RESPONSE 3 //receiving response
+#define S8P_COMMAND 4 //waiting for execution of command
+#define S8P_ACKNOWLEDGE 5 //sending acknowledge
+#define S8P_CLOSE 6 //close connection
 
 //table with S8P connections
 struct S8pConnection
@@ -109,11 +107,7 @@ void S8pConnect( unsigned char ConnNo )
   if( i < count( S8pConns ) )
   {
     //free this entry
-    if( S8pConns[i].State == S8P_LOCKED ||
-        S8pConns[i].State == S8P_LOCKED_FREE )
-      S8pConns[i].State = S8P_LOCKED_FREE;
-    else
-      S8pConns[i].State = S8P_FREE;
+    S8pConns[i].State = S8P_FREE;
     return;
   }
 
@@ -126,6 +120,8 @@ void S8pConnect( unsigned char ConnNo )
   //get pointer to connection
   pConn = &S8pConns[i];
 
+  debug_s8p_printf( "accept no=%u", ConnNo );
+
   //put new connection into table
   pConn->State = S8P_NEW;
   pConn->ConnNo = ConnNo;
@@ -148,12 +144,10 @@ void S8pClose( unsigned char ConnNo )
   //get pointer to connection
   pConn = &S8pConns[i];
 
+  debug_s8p_printf( "close no=%u", ConnNo );
+
   //drop connection from table
-  if( pConn->State == S8P_LOCKED ||
-      pConn->State == S8P_LOCKED_FREE )
-    pConn->State = S8P_LOCKED_FREE;
-  else
-    pConn->State = S8P_FREE;
+  pConn->State = S8P_FREE;
 }
 
 //called when sending data is possible
@@ -178,11 +172,10 @@ unsigned short S8pSend( unsigned char ConnNo, unsigned long Pos, unsigned char *
   switch( pConn->State )
   {
 
-    case S8P_LOCKED:
-    case S8P_LOCKED_FREE:
     case S8P_NEW:
     case S8P_RESPONSE:
     case S8P_COMMAND:
+      debug_s8p_printf( "send no=%u nothing", ConnNo );
       return 0; //send nothing
 
     case S8P_CHALLENGE: //sending challenge
@@ -190,6 +183,7 @@ unsigned short S8pSend( unsigned char ConnNo, unsigned long Pos, unsigned char *
         return 0; //send nothing
       len = min( sizeof( pConn->Buffer ) - Pos, MaxLen ); //send (part of) challenge
       memcpy( pBuffer, pConn->Buffer + Pos, len );
+      debug_s8p_printf( "send no=%u challenge pos=%lu len=%u", ConnNo, Pos, len );
       return len;
 
     case S8P_ACKNOWLEDGE: //sending acknowledge
@@ -199,6 +193,7 @@ unsigned short S8pSend( unsigned char ConnNo, unsigned long Pos, unsigned char *
         return 0; //send nothing
       len = min( sizeof( pConn->Buffer ) * 2 - Pos, MaxLen ); //send (part of) acknowledge
       memcpy( pBuffer, pConn->Buffer + Pos - sizeof( pConn->Buffer ), len );
+      debug_s8p_printf( "send no=%u acknowledge pos=%lu len=%u", ConnNo, Pos - sizeof( pConn->Buffer ), len );
       return len;
 
     case S8P_CLOSE:
@@ -267,6 +262,7 @@ unsigned short S8pReceived( unsigned char ConnNo, unsigned long Pos, unsigned ch
   {
 
     case S8P_RESPONSE: //receiving response
+      debug_s8p_printf( "recv no=%u response pos=%lu len=%u", ConnNo, Pos, Len );
       if( Pos < sizeof( pConn->Buffer ) ) //not behind response
         memcpy( pConn->Buffer + Pos, pBuffer, min( Len, sizeof( pConn->Buffer ) - Pos ) );
       if( Pos + Len >= sizeof( pConn->Buffer ) ) //entire response is received
@@ -312,22 +308,17 @@ static void S8pCommand( unsigned char Command[8], unsigned char Status[8] )
       //command: 0x00 7x<unused>
       //status: 0x00 <state> 6x<unused>
       Status[0] = 0x00;
-      cli( );
       Status[1] = OutputGetState( );
-      sei( );
       memset( Status + 2, 0x00, sizeof( Status ) - 2 );
       break;
 
     //change output
-    //get output
     case 0x01:
       //command: 0x01 <action> <output> 5x<unused>
-      cli( );
       OutputChange( Command[1], Command[2] );
       //status: 0x01 <state> 6x<unused>
       Status[0] = 0x01;
       Status[1] = OutputGetState( );
-      sei( );
       memset( Status + 2, 0x00, sizeof( Status ) - 2 );
       break;
 
@@ -342,12 +333,11 @@ static void S8pCommand( unsigned char Command[8], unsigned char Status[8] )
 void S8pTask( void ) //(extern)
 {
   static unsigned char conn = -1; //used to start serching at another connection each time
-  unsigned char i, state, req_send, Command[8];
+  unsigned char i, req_send, Command[8];
   struct S8pConnection * pConn;
   uint64_t xteaCtx;
 
   //search next S8P connection that has work to be done
-  cli( );
   for( i = 0; i < count( S8pConns ); i++ )
   {
     conn++; //next connection
@@ -358,26 +348,18 @@ void S8pTask( void ) //(extern)
       break;
   }
   if( i >= count( S8pConns ) ) //found none
-  {
-    sei( );
     return;
-  }
-  pConn = &S8pConns[conn]; //ger pointer to connection
-  state = pConn->State; //remember state
-  pConn->State = S8P_LOCKED; //lock connection
-  sei( );
+  pConn = &S8pConns[i]; //get pointer to connection
 
   //different states
   req_send = 0;
-  switch( state )
+  switch( pConn->State )
   {
 
     //new connection, waiting for challenge to be generated
     case S8P_NEW:
       //generate random data: d
-      cli( );
       RandomGetData( pConn->Random, sizeof( pConn->Random ) );
-      sei( );
       //generate challenge: c = ENC_k( ("BitAmEth" XOR d) . d )
       for( i = 0; i < sizeof( pConn->Random ); i++ )
         pConn->Buffer[i] = ((unsigned char *)"BitAmEth")[i] ^ pConn->Random[i];
@@ -387,7 +369,7 @@ void S8pTask( void ) //(extern)
                 (uint64_t *)pConn->Buffer, (uint64_t *)pConn->Buffer,
                 sizeof( pConn->Buffer ) / sizeof( uint64_t ), &xteaCtx );
       //now send challenge
-      state = S8P_CHALLENGE;
+      pConn->State = S8P_CHALLENGE;
       req_send = 1;
       break;
 
@@ -405,13 +387,33 @@ void S8pTask( void ) //(extern)
       //close connection if not
       if( i < sizeof( pConn->Random ) )
       {
-        state = S8P_CLOSE;
+        pConn->State = S8P_CLOSE;
         req_send = 1;
         break;
       }
       //execute command o2, let s be the status returned by this command
       memcpy( Command, pConn->Buffer + sizeof( pConn->Random ), sizeof( Command ) );
+      debug_s8p_printf( "command no=%u %02X %02X %02X %02X %02X %02X %02X %02X",
+                        pConn->ConnNo,
+                        pConn->Buffer[sizeof( pConn->Random ) + 0],
+                        pConn->Buffer[sizeof( pConn->Random ) + 1],
+                        pConn->Buffer[sizeof( pConn->Random ) + 2],
+                        pConn->Buffer[sizeof( pConn->Random ) + 3],
+                        pConn->Buffer[sizeof( pConn->Random ) + 4],
+                        pConn->Buffer[sizeof( pConn->Random ) + 5],
+                        pConn->Buffer[sizeof( pConn->Random ) + 6],
+                        pConn->Buffer[sizeof( pConn->Random ) + 7] );
       S8pCommand( Command, pConn->Buffer + sizeof( pConn->Random ) );
+      debug_s8p_printf( "status no=%u %02X %02X %02X %02X %02X %02X %02X %02X",
+                        pConn->ConnNo,
+                        pConn->Buffer[sizeof( pConn->Random ) + 0],
+                        pConn->Buffer[sizeof( pConn->Random ) + 1],
+                        pConn->Buffer[sizeof( pConn->Random ) + 2],
+                        pConn->Buffer[sizeof( pConn->Random ) + 3],
+                        pConn->Buffer[sizeof( pConn->Random ) + 4],
+                        pConn->Buffer[sizeof( pConn->Random ) + 5],
+                        pConn->Buffer[sizeof( pConn->Random ) + 6],
+                        pConn->Buffer[sizeof( pConn->Random ) + 7] );
       //generate acknowledge: a = ENC_k( ("Status  " XOR d) . s )
       for( i = 0; i < sizeof( pConn->Random ); i++ )
         pConn->Buffer[i] = ((unsigned char *)"Status  ")[i] ^ pConn->Random[i];
@@ -420,22 +422,14 @@ void S8pTask( void ) //(extern)
                 (uint64_t *)pConn->Buffer, (uint64_t *)pConn->Buffer,
                 sizeof( pConn->Buffer ) / sizeof( uint64_t ), &xteaCtx );
       //now send acknowledge
-      state = S8P_ACKNOWLEDGE;
+      pConn->State = S8P_ACKNOWLEDGE;
       req_send = 1;
       break;
 
-  } //switch( state )
+  } //switch( pConn->State )
 
-  //unlock connection
-  cli( );
-  if( pConn->State == S8P_LOCKED_FREE ) //connection freed by interrupt while locked
-    pConn->State = S8P_FREE; //free it now
-  else
-  {
-    pConn->State = state; //restore original (or modified) state
-    if( req_send ) //request to send next packet
-      TcpSend( pConn->ConnNo );
-  }
-  sei( );
+  //request to send next packet
+  if( req_send )
+    TcpSend( pConn->ConnNo );
 }
 
