@@ -1,6 +1,6 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
@@ -9,6 +9,7 @@
 
 #include "config.h"
 #include "checksum.h"
+#include "debug.h"
 #include "ethernet.h"
 #include "http.h"
 #include "ip.h"
@@ -26,7 +27,7 @@
 #define TCP_FIN 0x01
 #define TCP_FLAGS 0x3F
 
-#define TcpResendTicks 5 //time after which to resend a packet not ACKed (in 200ms steps ,max. 255)
+#define TcpResendTicks 5 //time after which to resend a packet not ACKed (in 200ms steps, max. 255)
 #define TcpTimeWaitTicks 20 //time to wait in TIME_WAIT state (in 200ms steps, max. 255)
 #define TcpTimeoutTicks 50 //maximum idle time of connection before it is reset (in 200ms steps, max. 255)
 #define TcpMaxLifeTimeTicks 150 //maximum lifetime of connection before it is reset (in 200ms steps, max. 255)
@@ -47,7 +48,6 @@ void TcpInit( void ) //(extern)
 //send a TCP packet
 //pData must point to a struct TcpPacket with TcpHdr.SrcPort, TcpHdr.DestPort,
 //TcpHdr.SeqNo, TcpHdr.AckNo, TcpHdr.WndSz and IpHdr.Dest already initialized
-//must be called with interrupts disabled
 static void TcpSendPacket( unsigned char * pData, unsigned short Length, unsigned char optLen, unsigned char flags )
 {
   struct TcpPacket * pTcpPack;
@@ -61,6 +61,17 @@ static void TcpSendPacket( unsigned char * pData, unsigned short Length, unsigne
   //(this saves us from always casting pData)
   pTcpPack = (struct TcpPacket *)pData;
 
+  debug_tcp_printf( "send src=%u dest=%u flags=%s%s%s%s%s%s len=%u",
+                    ntohs( pTcpPack->TcpHdr.SrcPort ),
+                    ntohs( pTcpPack->TcpHdr.DestPort ),
+                    flags & TCP_URG ? "U" : "",
+                    flags & TCP_ACK ? "A" : "",
+                    flags & TCP_PSH ? "P" : "",
+                    flags & TCP_RST ? "R" : "",
+                    flags & TCP_SYN ? "S" : "",
+                    flags & TCP_FIN ? "F" : "",
+                    Length );
+
   //fill in header values
   pTcpPack->TcpHdr.Ofs_Flags = htons( (unsigned short)((optLen + 23) & 0x3C) << 10 | (unsigned short)(flags & TCP_FLAGS) );
   pTcpPack->TcpHdr.Chk = 0x0000;
@@ -146,15 +157,20 @@ static void TcpSendDataSegment( unsigned char connNo, struct TcpConnection * pCo
     //get data to send from user
     if( sendDataLen > 0 )
     {
+      debug_tcp_printf( "notify send no=%u pos=%lu max_len=%u",
+                        connNo, pConn->SndNxt - (pConn->Iss + 1), sendDataLen );
       len = pConn->Notify->Send( connNo, pConn->SndNxt - (pConn->Iss + 1), Packet.Data, sendDataLen );
       if( len > sendDataLen ) //returned 0xFFFF (or any number too large) to close connection
       {
+        debug_tcp_printf( "notify send no=%u close", connNo );
         TcpEmptySegment( pConn, pConn->SndNxt, pConn->RcvNxt, TCP_FIN | TCP_ACK ); //send TCP FIN
         pConn->SndNxt++;
         pConn->State = TCP_FIN_WAIT_1; //go to state FIN-WAIT-1
+        debug_tcp_printf( "notify close no=%u", connNo );
         pConn->Notify->Close( connNo ); //signal "connection closed" to user
         return;
       }
+      debug_tcp_printf( "notify send no=%u len=%u", connNo, len );
       sendDataLen = len;
     }
   }
@@ -179,8 +195,7 @@ static void TcpSendDataSegment( unsigned char connNo, struct TcpConnection * pCo
   }
 }
 
-//tick procedure - call every 200ms from timer interrupt
-//must be called with interrupts disabled
+//tick procedure - call every 200ms
 void TcpTick200( void ) //(extern)
 {
   unsigned char i, MaxTicks;
@@ -195,7 +210,7 @@ void TcpTick200( void ) //(extern)
       TcpConns[i].Ticks++;
 
       //get maximum value for normal timer
-      MaxTicks = TcpConns[i].State == TCP_TIME_WAIT ? TcpResendTicks : TcpTimeWaitTicks;
+      MaxTicks = TcpConns[i].State == TCP_TIME_WAIT ? TcpTimeWaitTicks : TcpResendTicks;
       //normal timer elapsed
       if( TcpConns[i].Ticks >= MaxTicks )
       {
@@ -267,6 +282,7 @@ void TcpTick200( void ) //(extern)
           case TCP_ESTAB:
           case TCP_CLOSE_WAIT:
             TcpConns[i].State = TCP_CLOSED; //close connection
+            debug_tcp_printf( "notify close no=%u", i );
             TcpConns[i].Notify->Close( i ); //tell user that connection was closed
             break;
           default:
@@ -295,6 +311,7 @@ void TcpTick200( void ) //(extern)
           case TCP_ESTAB:
           case TCP_CLOSE_WAIT:
             TcpConns[i].State = TCP_CLOSED; //close connection
+            debug_tcp_printf( "notify close no=%u", i );
             TcpConns[i].Notify->Close( i ); //tell user that connection was closed
             break;
           default:
@@ -308,7 +325,6 @@ void TcpTick200( void ) //(extern)
 }
 
 //process a received TCP packet
-//must be called with interrupts disabled
 void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct TcpPacket * pTcpPack;
@@ -361,6 +377,17 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
   //get segment length (length of data in segment)
   len = Length - sizeof( struct EthernetHeader ) - sizeof( struct IpHeader ) - ofs;
 
+  debug_tcp_printf( "recv src=%u dest=%u flags=%s%s%s%s%s%s len=%u",
+                    ntohs( pTcpPack->TcpHdr.SrcPort ),
+                    ntohs( pTcpPack->TcpHdr.DestPort ),
+                    flags & TCP_URG ? "U" : "",
+                    flags & TCP_ACK ? "A" : "",
+                    flags & TCP_PSH ? "P" : "",
+                    flags & TCP_RST ? "R" : "",
+                    flags & TCP_SYN ? "S" : "",
+                    flags & TCP_FIN ? "F" : "",
+                    Length );
+
   //process options
   optLen = (unsigned char)(ofs - 20);
   optPtr = (unsigned char *)&pTcpPack->TcpHdr + sizeof( struct TcpHeader );
@@ -431,7 +458,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
       pNotify = &S8pNotify;
     }
     //HTTP
-    else if( localPort == ConfigWebPort )
+    else if( localPort == ConfigHttpPort )
     {
       rcvWnd = 256; //connection shall be accepted
       pNotify = &HttpNotify;
@@ -574,6 +601,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
         if( (long)(pConn->SndUna - pConn->Iss) > 0 ) //If SND.UNA > ISS (our SYN has been ACKed),
         {
           pConn->State = TCP_ESTAB; //change the connection state to ESTABLISHED,
+          debug_tcp_printf( "notify connect no=%u", connNo );
           pConn->Notify->Connect( connNo ); //signal "connected"
           TcpEmptySegment( pConn, pConn->SndNxt, pConn->RcvNxt, TCP_ACK ); //form an ACK segment and send it.
         }
@@ -661,6 +689,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 
       case TCP_SYN_RCVD:
         pConn->State = TCP_CLOSED; //enter the CLOSED state and delete the TCB, an return.
+        debug_tcp_printf( "notify close no=%u", connNo );
         pConn->Notify->Close( connNo ); //signal "connection refused"
         return;
 
@@ -669,6 +698,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
       case TCP_FIN_WAIT_2:
       case TCP_CLOSE_WAIT:
         pConn->State = TCP_CLOSED; //Enter the CLOSED state, delete the TCB, and return.
+        debug_tcp_printf( "notify close no=%u", connNo );
         pConn->Notify->Close( connNo ); //signal "connection reset"
         return;
 
@@ -699,6 +729,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
         // - this is always the case here, because SYN not in window would habe been removed above
         TcpEmptySegment( pConn, ack, 0, TCP_RST ); //send a reset,
         pConn->State = TCP_CLOSED; //enter the CLOSED state, delete the TCB, and return.
+        debug_tcp_printf( "notify close no=%u", connNo );
         pConn->Notify->Close( connNo ); //signal "connection reset"
         return;
 
@@ -725,6 +756,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
             (long)(ack - pConn->SndNxt) <= 0 )
         {
           pConn->State = TCP_ESTAB; //then enter ESTABLISHED state
+          debug_tcp_printf( "notify connect no=%u", connNo );
           pConn->Notify->Connect( connNo ); //signal "connected"
           //and continue processing.
         }
@@ -757,6 +789,8 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
           pConn->SndWl2 = ack; //and set SND.WL2 <- SEG.ACK.
         }
         pConn->Timeout = 0; //restart timeout timer
+        debug_tcp_printf( "notify sent no=%u pos=%lu",
+                          connNo, pConn->SndNxt - (pConn->Iss + 1) );
         pConn->Notify->Sent( connNo, pConn->SndUna - (pConn->Iss + 1) ); //signal "send completed" (up to ack)
         //additional processing
         switch( pConn->State )
@@ -767,7 +801,10 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
             //no break here
           case TCP_FIN_WAIT_2:
             if( pConn->SndUna == pConn->SndNxt ) //if the retransmission queue is empty,
+            {
+              debug_tcp_printf( "notify close no=%u", connNo );
               pConn->Notify->Close( connNo ); //the user's CLOSE can be acknowledged
+            }
             break;
           case TCP_CLOSING:
             if( pConn->SndUna == pConn->SndNxt ) //if the ACK acknowledges our FIN
@@ -812,9 +849,12 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
         pConn->RcvWnd -= len; //make receive window smaller (len <= pConn->RcvWnd)
         pConn->RcvNxt += len; //advance sequence number of next data to receive
         pConn->Timeout = 0; //restart timeout timer
+        debug_tcp_printf( "notify recv no=%u pos=%lu len=%u, min_wnd=%u",
+                           connNo, pConn->RcvNxt - (pConn->Irs + 1) - len, len, pConn->RcvWnd );
         pConn->RcvWnd = pConn->Notify->Received( connNo, pConn->RcvNxt - (pConn->Irs + 1) - len, //give received data to user
                                                  (unsigned char *)&pTcpPack->TcpHdr + ofs, len, //(update receive window size)
                                                  pConn->RcvWnd );
+        debug_tcp_printf( "notify recv no=%u wnd=%u", connNo, pConn->RcvWnd );
         sendAck = 1; //remember to send an ACK
         break;
 
@@ -837,6 +877,7 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
         TcpEmptySegment( pConn, pConn->SndNxt, pConn->RcvNxt, TCP_FIN | TCP_ACK ); //send a FIN
         pConn->SndNxt++;
         pConn->State = TCP_CLOSING;
+        debug_tcp_printf( "notify close no=%u", connNo );
         pConn->Notify->Close( connNo ); //signal "connection closed" to user
         return; //we do not need to send an ack, becaue we already sent it with our FIN
 
@@ -867,7 +908,6 @@ void TcpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 }
 
 //open a TCP connection
-//must be called with interrupts disabled
 //must not be called from a TCP notification function
 //returns the connection number of the new connection of 0xFF in case of error
 unsigned char TcpOpen( unsigned char * remoteIp, unsigned short remotePort, //(extern)
@@ -879,6 +919,10 @@ unsigned char TcpOpen( unsigned char * remoteIp, unsigned short remotePort, //(e
   unsigned short localPort;
   unsigned char i;
 
+  debug_tcp_printf( "open ip=%u.%u.%u.%u port=%u",
+                    remoteIp[0], remoteIp[1], remoteIp[2], remoteIp[3],
+                    remotePort );
+
   //search an unused local port
   for( localPort = nextLocalPort; ; localPort++ )
   {
@@ -922,17 +966,20 @@ unsigned char TcpOpen( unsigned char * remoteIp, unsigned short remotePort, //(e
   TcpConns[i].RcvWnd = initialWnd;
   TcpConns[i].Notify = Notify;
 
+  debug_tcp_printf( "open no=%u", i );
+
   //return connection number
   return i;
 }
 
 //close a TCP connection
-//must be called with interrupts disabled
 //must not be called from a TCP notification function
 void TcpClose( unsigned char connNo ) //(extern)
 {
   struct TcpConnection * pConn;
 
+  debug_tcp_printf( "close no=%u", connNo );
+
   //connection does not exist
   if( connNo >= count( TcpConns ) || TcpConns[connNo].State == TCP_CLOSED )
     return;
@@ -940,7 +987,7 @@ void TcpClose( unsigned char connNo ) //(extern)
   //get connection
   pConn = &TcpConns[connNo];
 
-  //different actions in different staes accroding to RFC793
+  //different actions in different states accroding to RFC793
   switch( pConn->State  )
   {
 
@@ -954,6 +1001,7 @@ void TcpClose( unsigned char connNo ) //(extern)
       TcpEmptySegment( pConn, pConn->SndNxt, pConn->RcvNxt, TCP_FIN | TCP_ACK ); //form a FIN segment and send it,
       pConn->SndNxt++;
       pConn->State = TCP_FIN_WAIT_1; //and enter FIN-WAIT-1 state;
+      debug_tcp_printf( "notify close no=%u", connNo );
       pConn->Notify->Close( connNo ); //signal "connection closed" to user
       break;
 
@@ -961,6 +1009,7 @@ void TcpClose( unsigned char connNo ) //(extern)
       TcpEmptySegment( pConn, pConn->SndNxt, pConn->RcvNxt, TCP_FIN | TCP_ACK ); //send a FIN segment,
       pConn->SndNxt++;
       pConn->State = TCP_CLOSING; //enter CLOSING state.
+      debug_tcp_printf( "notify close no=%u", connNo );
       pConn->Notify->Close( connNo ); //signal "connection closed" to user
       break;
 
@@ -968,13 +1017,14 @@ void TcpClose( unsigned char connNo ) //(extern)
 }
 
 //request sending on a TCP connection
-//must be called with interrupts disabled
 //must not be called from a TCP notification function
 //this makes the send notification to be called if possible
 void TcpSend( unsigned char connNo ) //(extern)
 {
   struct TcpConnection * pConn;
 
+  debug_tcp_printf( "send no=%u", connNo );
+
   //connection does not exist
   if( connNo >= count( TcpConns ) || TcpConns[connNo].State == TCP_CLOSED )
     return;
