@@ -1,17 +1,18 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
 
 #include <stdio.h>
 #include <avr/io.h>
+#include <avr/wdt.h>
 
 #include "macros.h"
 #include "uart.h"
 
-#define BAUD_RATE 9600
+#define BAUD_RATE 38400
 
 //initialize
 void UartInit( void ) //(extern)
@@ -29,11 +30,12 @@ void UartInit( void ) //(extern)
 int UartPutchar( char c ) //(extern)
 {
   //wait until last character was sent
-  while( bit_is_clear( UCSR0A, UDRE ) );
+  while( bit_is_clear( UCSR0A, UDRE ) )
+    wdt_reset( );
 
   //send character
   UDR0 = c;
 
-  return (0);
+  return 0;
 }
 
