@@ -1,12 +1,13 @@
 /* 8BitAmEthernet
- * version 0.2 date 2005-03-07
- * Copyright 2005 Stefan Schuermans <1stein@schuermans.info>
+ * version 0.3.1 date 2006-03-12
+ * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
  */
 
 #include "config.h"
 #include "checksum.h"
+#include "debug.h"
 #include "ethernet.h"
 #include "ip.h"
 #include "macros.h"
@@ -19,8 +20,7 @@ unsigned char UdpEchoTickCnt = 0; //tick counter
 #define UdpEchoReliesMax 3 //maximum value for UdpEchoReplies
 unsigned char UdpEchoReplies = 0; //number of UDP echo replies that may be sent at the moment
 
-//tick procedure - call every 200ms from timer interrupt
-//must be called with interrupts disabled
+//tick procedure - call every 200ms
 void UdpTick200( void ) //(extern)
 {
   //count ticks
@@ -37,7 +37,6 @@ void UdpTick200( void ) //(extern)
 }
 
 //process a received UDP echo packet
-//must be called with interrupts disabled
 static void UdpEchoRecv( unsigned char * pData, unsigned short Length )
 {
   struct UdpPacket * pUdpPack;
@@ -54,13 +53,18 @@ static void UdpEchoRecv( unsigned char * pData, unsigned short Length )
 
   //only reply with allowed packet rate
   if( UdpEchoReplies == 0 )
+  {
+    debug_udp_printf( "echo len=%u (dropped)", Length );
     return;
+  }
   UdpEchoReplies--;
 
+  debug_udp_printf( "echo len=%u", Length );
+
   //send an UDP echo
   // - use same buffer to send reply
-  // - this saves us from needing to allocate a new buffer
-  // - this saves us from needing to copy the data
+  // - this saves us from allocating a new buffer
+  // - this saves us from copying the data
   pUdpPack->UdpHdr.DestPort = pUdpPack->UdpHdr.SrcPort; //back to originationg port
   pUdpPack->UdpHdr.SrcPort = htons( 7 ); //UDP echo port
   ip_cpy( pUdpPack->IpHdr.Dest, pUdpPack->IpHdr.Src ); //back to originating IP
@@ -68,7 +72,6 @@ static void UdpEchoRecv( unsigned char * pData, unsigned short Length )
 }
 
 //process a received UDP packet
-//must be called with interrupts disabled
 void UdpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct UdpPacket * pUdpPack;
@@ -101,6 +104,11 @@ void UdpRecv( unsigned char * pData, unsigned short Length ) //(extern)
                 ntohs( pUdpPack->UdpHdr.Length ) ) != 0 )
     return;
 
+  debug_udp_printf( "recv src=%u dest=%u len=%u",
+                    ntohs( pUdpPack->UdpHdr.SrcPort ),
+                    ntohs( pUdpPack->UdpHdr.DestPort ),
+                    Length );
+
   //branch according to destination port
   switch( ntohs( pUdpPack->UdpHdr.DestPort ) )
   {
@@ -113,7 +121,6 @@ void UdpRecv( unsigned char * pData, unsigned short Length ) //(extern)
 
 //send an UDP packet
 //pData must point to a struct UdpPacket with UdpHdr.SrcPort, UdpHdr.DestPort and IpHdr.Dest already initialized
-//must be called with interrupts disabled
 void UdpSend( unsigned char * pData, unsigned short Length ) //(extern)
 {
   struct UdpPacket * pUdpPack;
@@ -127,6 +134,11 @@ void UdpSend( unsigned char * pData, unsigned short Length ) //(extern)
   //(this saves us from always casting pData)
   pUdpPack = (struct UdpPacket *)pData;
 
+  debug_udp_printf( "send src=%u dest=%u len=%u",
+                    ntohs( pUdpPack->UdpHdr.SrcPort ),
+                    ntohs( pUdpPack->UdpHdr.DestPort ),
+                    Length );
+
   //fill in header values
   pUdpPack->UdpHdr.Length = htons( Length - sizeof( struct EthernetHeader ) - sizeof( struct IpHeader ) );
   pUdpPack->UdpHdr.Chk = 0x0000;
